package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.slmbi.dao.InstrumentPanelMapper;
import com.syc.slm.slmbi.dto.QueryInstrumentPanelDTO;
import com.syc.slm.slmbi.dto.SaveInstrumentPanelDTO;
import com.syc.slm.slmbi.entity.InstrumentPanel;
import com.syc.slm.slmbi.entity.InstrumentPanelDraft;
import com.syc.slm.slmbi.entity.InstrumentPanelTree;
import com.syc.slm.slmbi.enums.AccessType;
import com.syc.slm.slmbi.enums.NodeType;
import com.syc.slm.slmbi.enums.PublishedStatus;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.*;
import com.syc.slm.slmbi.vo.InstrumentPanelConfigVo;
import com.syc.slm.slmbi.vo.InstrumentPanelVo;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Set;

/**
 * slm-bi
 * 仪表板
 *
 * @author : lin
 * @date : 2021-03-25 16:17
 **/
@Slf4j
@Service
public class InstrumentPanelServiceImpl extends ServiceImpl<InstrumentPanelMapper, InstrumentPanel>
    implements InstrumentPanelService {
    @Autowired
    private InstrumentPanelTreeService treeService;

    @Autowired
    private InstrumentPanelPermissionService permissionService;

    @Autowired
    private InstrumentPanelConfigService configService;

    @Autowired
    private InstrumentPanelDraftService draftService;

    @Override
    @Transactional(readOnly = true)
    public IPage<InstrumentPanelVo> selectInstrumentPanelList(String appId, QueryInstrumentPanelDTO where) {
        IPage<InstrumentPanelVo> page = new Page<>(where.getPageNum(), where.getPageSize());
        QueryWrapper<InstrumentPanel> wrapper = new QueryWrapper<>();
        wrapper.eq("p.app_id", appId);
        wrapper.eq("p.group_id", where.getGroupId());
        wrapper.eq("p.record_status", SlmConstants.DATA_VALID);
        if (StringUtils.isNotEmpty(where.getName())) {
            wrapper.like("p.name", where.getName());
        }
        if (StringUtils.isNotEmpty(where.getCreatorName())) {
            wrapper.like("p.creator_name", where.getCreatorName());
        }
        if (StringUtils.isNotEmpty(where.getUpdaterName())) {
            wrapper.like("p.updater_name", where.getUpdaterName());
        }
        if (where.getPublished() != null) {
            wrapper.eq("p.published", where.getPublished());
        }
        if (where.getUpdateTimeBegin() != null) {
            wrapper.gt("p.update_time", where.getUpdateTimeBegin());
        }
        if (where.getUpdateTimeEnd() != null) {
            wrapper.lt("p.update_time", where.getUpdateTimeEnd());
        }
        return baseMapper.selectInstrumentPanelList(page, wrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String savePanel(String appId, SaveInstrumentPanelDTO panel) {
        log.info("保存仪表板");
        return savePanelData(appId, panel);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchPublish(List<String> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new SysException("请选择你要操作的数据");
        }
        List<InstrumentPanel> panels = baseMapper.selectBatchIds(ids);
        if (CollectionUtils.isNotEmpty(panels)) {
            panels.forEach(x -> {
                x.setPublished(PublishedStatus.PUBLISHED.value);
                baseMapper.updateById(x);
            });
            log.info("修改配置状态为发布状态");
            configService.publishConfigByPanelIds(ids);
            log.info("修改权限状态为发布状态");
            permissionService.publishPermissionByPanelIds(ids);
            log.info("复制一份草稿");
            draftService.saveDrafts(panels);
        }

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String delPanel(String id) {
        if (StringUtils.isEmpty(id)) {
            throw new SysException("请选择你要操作的数据");
        }
        InstrumentPanel panel = baseMapper.selectById(id);
        if (SlmConstants.DATA_DELETED.equals(panel.getRecordStatus())) {
            throw new SysException("该数据已被删除，请刷新页面");
        }
        panel.setRecordStatus(SlmConstants.DATA_DELETED);
        baseMapper.updateById(panel);

        log.info("删除该仪表板下的配置");
        delAllSetting(id);
        log.info("删除仪表板草稿表");
        InstrumentPanelDraft draft = new InstrumentPanelDraft();
        draft.setRecordStatus(SlmConstants.DATA_DELETED);
        draft.setId(id);
        draftService.updateById(draft);
        return id;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveAsPanel(String appId, SaveInstrumentPanelDTO panel) {
        log.info("仪表板另存为");
        return savePanelData(appId, panel);
    }

    @Override
    @Transactional(readOnly = true)
    public Boolean showRecoveryBtn(String id) {
        return draftService.getById(id) != null;
    }

    @Override
    @Transactional(readOnly = true)
    public InstrumentPanelConfigVo recoveryPanel(String id) {
        return configService.getPublishConfigByPanelId(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String offline(String id) {
        if (StringUtils.isBlank(id)) {
            throw new SysException("请选择你要操作的数据");
        }
        InstrumentPanel panel = baseMapper.selectById(id);
        if (ObjectUtils.isEmpty(panel)) {
            throw new SysException("该数据已不存在，请刷新页面");
        }
        if (SlmConstants.DATA_DELETED.equals(panel.getRecordStatus())) {
            throw new SysException("该数据已被删除，请刷新页面");
        }
        panel.setPublished(PublishedStatus.OFFLINE.value);
        // panel.setRecordStatus(SlmConstants.DATA_DELETED);
        baseMapper.updateById(panel);
        //        log.info("删除配置");
        //        log.info("删除权限");
        //        log.info("删除草稿");
        //        log.info("删除仪表板树");
        //        delAllSetting(id);
        return id;
    }

    @Override
    public void batchUpdatePanelByIds(Set<String> panelIds, Integer recordStatus) {
        if (CollectionUtils.isNotEmpty(panelIds)) {
            baseMapper.batchUpdatePanelByIds(panelIds, recordStatus);
        }
    }

    @Override
    public void checkName(@NonNull String appId, String panelId, @NonNull String panelName) {
        LambdaQueryWrapper<InstrumentPanel> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(InstrumentPanel::getName, panelName);
        wrapper.eq(InstrumentPanel::getAppId, appId);
        wrapper.eq(InstrumentPanel::getRecordStatus, SlmConstants.DATA_VALID);
        if (StringUtils.isNotEmpty(panelId)) {
            wrapper.eq(InstrumentPanel::getId, panelId);
        }
        if (baseMapper.selectCount(wrapper) > 0) {
            throw new SysException("仪表板名字:" + panelName + "已存在!");
        }
    }

    private String savePanelData(String appId, SaveInstrumentPanelDTO panel) {
        if (StringUtils.isEmpty(panel.getName())) {
            throw new SysException("仪表板名字必填");
        }
        if (StringUtils.isEmpty(panel.getGroupId())) {
            throw new SysException("仪表板分组必填");
        }
        log.info("检查仪表板名称是否重复");
        this.checkName(appId, null, panel.getName());

        log.info("保存仪表板访问权限");
        String panelId = IdWorker.get32UUID();
        String accessType = AccessType.PUBLIC.value;
        if (ObjectUtils.isNotEmpty(panel.getPermission()) && StringUtils.isNotEmpty(panel.getPermission()
                                                                                        .getAccessType())) {
            accessType = panel.getPermission().getAccessType();
            permissionService.savePanelPermission(panel.getPermission().getCustomPermissions(),
                                                  panelId,
                                                  panel.getPermission().getAccessType(),
                                                  PublishedStatus.WAIT_PUBLISHED.value);

        }

        InstrumentPanel instrumentPanel = new InstrumentPanel(panelId,
                                                              panel.getName(),
                                                              panel.getGroupId(),
                                                              accessType,
                                                              PublishedStatus.WAIT_PUBLISHED.value);

        log.info("保存仪表板配置");
        if (StringUtils.isNotEmpty(panel.getConfigDetails())) {
            configService.savePanelConfig(panelId, panel.getConfigDetails(), PublishedStatus.WAIT_PUBLISHED.value);
        }

        log.info("初始化仪表板树");
        treeService.initTree(instrumentPanel.getGroupId(),
                             NodeType.PANEL.value,
                             instrumentPanel.getId(),
                             instrumentPanel.getName());

        if (CollectionUtils.isNotEmpty(panel.getAssemblys())) {
            InstrumentPanelTree tree = treeService.getById(instrumentPanel.getId());
            log.info("保存仪表板组件树");
            treeService.savePanelAssemblyTree(tree, panel.getAssemblys(), instrumentPanel.getId());
        } else {
            log.info("前端没有构建组件树");
        }
        log.info("保存仪表板数据");
        baseMapper.insert(instrumentPanel);
        return panelId;
    }

    private void delAllSetting(String id) {
        log.info("删除仪表板对应权限");
        permissionService.delPermissionByPanelId(id, null);

        log.info("删除仪表板对应配置");
        configService.delConfig(id, null);

        log.info("删除仪表板树");
        treeService.delTreeByPanelId(id);
    }

}
